// CTexture.cpp
// by Comanche

#include "CTexture.h"
#include "CLog.h"

extern CLog Log;

//////////////////////////////////////////////////////////////////////////////////

CTexture::CTexture(char *filename)
{
	tfh = OpenFile(filename,&file_data,OF_READ);

	int i=0;
	while(filename[i]!=0)
	{		
		if(filename[i]=='\\')
		{
			i++;
			break;
		}
		i++;
	}
	strcpy(Name,&filename[i]);
	
	if(!strcmp(filename,""))
	{
		isfailure=TRUE;
		return;
	}

	if(tfh==HFILE_ERROR)
	{
		isfailure=TRUE;
		sprintf(buffer,"> Error: Reading %s (doesn't exist)",filename);
		Log.WriteErrorToLogFile(buffer);
		return;
	}

	_lread(tfh,&bitmap.bitmapfileheader,sizeof(BITMAPFILEHEADER));

	if (bitmap.bitmapfileheader.bfType!=BITMAP_ID)
	{
		isfailure=TRUE;
		sprintf(buffer,"> Error: Reading %s (not a bitmap)",filename);
		Log.WriteErrorToLogFile(buffer);
		_close(tfh);
		return;
	}

	_lread(tfh,&bitmap.bitmapinfoheader,sizeof(BITMAPINFOHEADER));

	if (bitmap.bitmapinfoheader.biBitCount!=8 &&
		bitmap.bitmapinfoheader.biBitCount!=24)
	{
		isfailure=TRUE;
		sprintf(buffer,"> Error: Reading %s (not 8 or 24 bit)",filename);
		Log.WriteErrorToLogFile(buffer);
		_close(tfh);
		return;
	}

	if (bitmap.bitmapinfoheader.biWidth>256 ||
		bitmap.bitmapinfoheader.biHeight>256)
	{
		isfailure=TRUE;
		sprintf(buffer,"> Error: Reading %s (over 256x256)",filename);
		Log.WriteErrorToLogFile(buffer);
		_close(tfh);
		return;
	}	
	
	if(bitmap.bitmapinfoheader.biWidth%32!=0 &&
		bitmap.bitmapinfoheader.biHeight%32!=0)
	{
		isfailure=TRUE;
		sprintf(buffer,"> Error: Reading %s",filename);
		Log.WriteErrorToLogFile(buffer);
		_close(tfh);
		return;
	}

	if(bitmap.bitmapinfoheader.biCompression!=BI_RGB)
	{
		isfailure=TRUE;
		sprintf(buffer,"> Error: Reading %s (is compressed)",filename);
		Log.WriteErrorToLogFile(buffer);
		_close(tfh);
		return;
	}

	if (bitmap.bitmapinfoheader.biBitCount==8)
	{
		int NumColor=256;
		if(bitmap.bitmapinfoheader.biClrUsed!=0)
			NumColor=bitmap.bitmapinfoheader.biClrUsed;
		_lread(tfh, &bitmap.palette,NumColor*sizeof(PALETTEENTRY));
		for (int index=0; index < NumColor; index++)
		{
			int temp_color = bitmap.palette[index].peRed;
			bitmap.palette[index].peRed  = bitmap.palette[index].peBlue;
			bitmap.palette[index].peBlue = temp_color;
       		bitmap.palette[index].peFlags = PC_NOCOLLAPSE;
		}
	}

	_lseek(tfh,-(int)(bitmap.bitmapinfoheader.biSizeImage),SEEK_END);
 
    if (!(bitmap.buffer = (UCHAR *)malloc(bitmap.bitmapinfoheader.biSizeImage)))
    {
		sprintf(buffer,"> Error: Reading %s",filename);
		Log.WriteErrorToLogFile(buffer);
		_close(tfh);
		return;
    }

	_lread(tfh,bitmap.buffer,bitmap.bitmapinfoheader.biSizeImage);	
	
	width = bitmap.bitmapinfoheader.biWidth;
	height = bitmap.bitmapinfoheader.biHeight;
	bitcount = bitmap.bitmapinfoheader.biBitCount;

	isfailure=FALSE;
	_close(tfh);
}

//////////////////////////////////////////////////////////////////////////////////

CTexture::~CTexture()
{
	free(bitmap.buffer);
}

//////////////////////////////////////////////////////////////////////////////////

RGBTRIPLE CTexture::GetPixel(int x, int y)
{
	RGBTRIPLE color;
	
	if(bitcount==24)
	{
		int pos=(x*3)+((height-y-1)*width*3);
		color.rgbtRed=bitmap.buffer[pos+2];
		color.rgbtGreen=bitmap.buffer[pos+1];
		color.rgbtBlue=bitmap.buffer[pos];
		return(color);
	}
	else if(bitcount==8)
	{
		int pos=x+((height-y-1)*width);
		BYTE index=bitmap.buffer[pos];
		color.rgbtRed=bitmap.palette[index].peRed;
		color.rgbtGreen=bitmap.palette[index].peGreen;
		color.rgbtBlue=bitmap.palette[index].peBlue;
		return(color);
	}
	color.rgbtBlue=255;		// Dummies for Compiler
	color.rgbtGreen=255;
	color.rgbtRed=255;
	return(color);
}


